@@ -20,7 +20,11 @@ import sys
 import math
 import zlib
 import base64
-from cStringIO import StringIO
+import types
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
 
 
 try:
@@ -178,27 +182,46 @@ def exP1multiN(pdf, newPageSize, n):
     return buf2
 
 
-def generateNup(inPath, n, outputPat=None, dirs="RD"):
+def isFileLike(obj):
+    "Is this a file-like object?"
+    
+    if type(obj) == types.FileType:
+        return True
+    if set("read seek close".split()).issubset(set(dir(obj))):
+        return True
+
+    return False
+    
+    
+def generateNup(inPathOrFile, n, outputPat=None, dirs="RD", verbose=False):
     "Generate a N-up document version."    
 
     assert isSquare(n) or isHalfSquare(n)
+    if isFileLike(inPathOrFile):
+        assert outputPat is not None
 
     # derive the ouput path from the output pattern, if given
-    if not outputPat:
-        outputPat = "%(dirname)s/%(base)s-%(n)dup%(ext)s"
-    dirname = os.path.dirname(inPath) or "."
-    basename = os.path.basename(inPath)
-    base = os.path.basename(os.path.splitext(inPath)[0])
-    ext = os.path.splitext(inPath)[1]    
-    aDict = {
-        "dirname":dirname, "basename":basename, 
-        "base":base, "n":n, "ext":ext
-    }
-    outPath = outputPat % aDict
-    outPath = os.path.normpath(outPath)
+    if type(inPathOrFile) in types.StringTypes:
+        if not outputPat:
+            outputPat = "%(dirname)s/%(base)s-%(n)dup%(ext)s"
+        dirname = os.path.dirname(inPathOrFile) or "."
+        basename = os.path.basename(inPathOrFile)
+        base = os.path.basename(os.path.splitext(inPathOrFile)[0])
+        ext = os.path.splitext(inPathOrFile)[1]    
+
+        aDict = {
+            "dirname":dirname, "basename":basename, 
+            "base":base, "n":n, "ext":ext
+        }
+        outPath = outputPat % aDict
+        outPath = os.path.normpath(outPath)
+        f = open(inPathOrFile, "rb")
+    elif isFileLike(inPathOrFile):
+        f = inPathOrFile
+        outPath = outputPat % {"n":n}
 
     # get info about source document
-    docReader = PdfFileReader(open(inPath, "rb"))
+    docReader = PdfFileReader(f)
     numPages = docReader.getNumPages()
     oldPageSize = docReader.getPage(0).mediaBox.upperRight
 
@@ -211,7 +234,7 @@ def generateNup(inPath, n, outputPat=None, dirs="RD"):
     
     buf = exP1multiN(_mtA4Pdf, newPageSize, np)
     open(outPath, "wb").write(buf.read())
-    
+ 
     # calculate mini page areas
     rects = calcRects(newPageSize, n, dirs)
         
@@ -221,10 +244,14 @@ def generateNup(inPath, n, outputPat=None, dirs="RD"):
     for i in range(numPages):
         if i % n == 0:
             newPageNum += 1
-        op = (inPath, i, (0, 0, None, None), i/n, rects[i % n])
+        op = (inPathOrFile, i, (0, 0, None, None), i/n, rects[i % n])
         ops.append(op)
 
-    srcr = srcReader = PdfFileReader(open(inPath, "rb"))
+    if type(inPathOrFile) in types.StringTypes:
+        f = open(inPathOrFile, "rb")
+    elif isFileLike(inPathOrFile):
+        f = inPathOrFile
+    srcr = srcReader = PdfFileReader(f)
     srcPages = [srcr.getPage(i) for i in range(srcr.getNumPages())]
 
     outr = outReader = PdfFileReader(open(outPath, "rb"))
@@ -243,7 +270,7 @@ def generateNup(inPath, n, outputPat=None, dirs="RD"):
 
     for destPageNum, ops in mapping.items():
         for op in ops:
-            inPath, srcPageNum, srcRect, destPageNum, destRect = op
+            inPathOrFile, srcPageNum, srcRect, destPageNum, destRect = op
             page2 = srcPages[srcPageNum]
             page1 = outPages[destPageNum]
             pageWidth, pageHeight = page2.mediaBox.upperRight
@@ -281,7 +308,10 @@ def generateNup(inPath, n, outputPat=None, dirs="RD"):
             page2Content.operations.insert(0, [[], "q"])
             
             # handle rotation
-            rotation = page2["/Rotate"].getObject()
+            try:
+                rotation = page2["/Rotate"].getObject()
+            except KeyError:
+                rotation = 0
             if rotation in (180, 270):
                 dw, dh = destWidth, destHeight
                 arr = [-xScale, 0, 0, -yScale, destX+dw, destY+dh]
@@ -302,6 +332,7 @@ def generateNup(inPath, n, outputPat=None, dirs="RD"):
 
     outputStream = open(outPath, "wb")
     output.write(outputStream)
-    outputStream.close()    
+    outputStream.close()
     
-    print "written: %s" % outPath
+    if verbose:
+        print "written: %s" % outPath
