@@ -8,6 +8,10 @@ import os
 import sys
 import math
 import unittest
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
 
 try:
     from pyPdf import PdfFileReader, PdfFileWriter
@@ -42,7 +46,7 @@ class LayoutingTests(unittest.TestCase):
             for n in (4, 9, 16):
                 outName = os.path.splitext(path0)[0] + "-%dup.pdf" % n
                 path1 = os.path.join(".", outName)
-                generateNup(path0, n, path1) # , dirs="UL")
+                generateNup(path0, n, path1, verbose=True) # , dirs="UL")
     
                 # assert output has correct number of pages
                 input = PdfFileReader(file(path0, "rb"))
@@ -66,7 +70,7 @@ class LayoutingTests(unittest.TestCase):
             for n in (2, 8, 18):
                 outName = os.path.splitext(path0)[0] + "-%dup.pdf" % n
                 path1 = os.path.join(".", outName)
-                generateNup(path0, n, path1) # , dirs="UL")
+                generateNup(path0, n, path1, verbose=True) # , dirs="UL")
     
                 # assert output has correct number of pages
                 input = PdfFileReader(file(path0, "rb"))
@@ -92,7 +96,7 @@ class LayoutingTests(unittest.TestCase):
             for n in (2, 4, 8, 9):
                 outName = os.path.splitext(path0)[0] + "-%dup.pdf" % n
                 path1 = os.path.join(".", outName)
-                generateNup(path0, n, path1) # , dirs="UL")
+                generateNup(path0, n, path1, verbose=True) # , dirs="UL")
     
                 # assert output has correct number of pages
                 input = PdfFileReader(file(path0, "rb"))
@@ -127,7 +131,7 @@ class RotationTests(unittest.TestCase):
         output.write(outputStream)
         outputStream.close()
         for j in (2, 4, 8, 9): 
-            generateNup(outPath, j)
+            generateNup(outPath, j, verbose=True)
 
 
     def test1(self):
@@ -145,7 +149,33 @@ class RotationTests(unittest.TestCase):
         output.write(outputStream)
         outputStream.close()
         for j in (2, 4, 8, 9): 
-            generateNup(outPath, j)
+            generateNup(outPath, j, verbose=True)
+
+
+class InMemoryTests(unittest.TestCase):
+    "Test with an input document in memory."
+
+    def test0(self):
+        "Test using file object as input document."
+
+        n = 4
+        path0  = "samples/test-a4-l.pdf"
+        f = open(path0, "rb")
+        outName = os.path.splitext(path0)[0] + "-%dup-fromFile.pdf" % n
+        path1 = os.path.join(".", outName)
+        generateNup(f, n, path1, verbose=True)
+
+
+    def test1(self):
+        "Test using StringIO object input document."
+
+        n = 4
+        path0  = "samples/test-a4-l.pdf"
+        pdfCode = open(path0, "rb").read()
+        f = StringIO(pdfCode)
+        outName = os.path.splitext(path0)[0] + "-%dup-fromStringIO.pdf" % n
+        path1 = os.path.join(".", outName)
+        generateNup(f, n, path1, verbose=True)
 
 
 if __name__ == "__main__":
